@@ -692,7 +692,7 @@ def RemoveLabels(LabelImage, minZ = 2):
                     LabelImage[LabelImage == regionlabel] = 0
     return LabelImage                
 
-def STARPrediction3D(image, model, n_tiles, MaskImage = None, smartcorrection = None, UseProbability = True, globalthreshold = 1.0E-5, min_size = 100):
+def STARPrediction3D(image, model, n_tiles, MaskImage = None, smartcorrection = None, UseProbability = True, globalthreshold = 1.0E-5, min_size = 100, extent = 0):
     
     copymodel = model
     image = normalize(image, 1, 99.8, axis = (0,1,2))
@@ -740,7 +740,7 @@ def STARPrediction3D(image, model, n_tiles, MaskImage = None, smartcorrection =
 
     
     print('Doing Watershedding')      
-    Watershed, Markers = WatershedwithMask3D(MaxProjectDistance.astype('uint16'), StarImage.astype('uint16'), MaskImage.astype('uint16'), grid )
+    Watershed, Markers = WatershedwithMask3D(MaxProjectDistance.astype('uint16'), StarImage.astype('uint16'), MaskImage.astype('uint16'), grid, extent )
     Watershed = fill_label_holes(Watershed.astype('uint16'))
   
        
@@ -769,33 +769,35 @@ def VetoRegions(Image, Zratio = 3):
 
 #Default method that works well with cells which are below a certain shape and do not have weak edges
     
-def iou3D(boxA, centroid):
+def iou3D(boxA, centroid, extent = 0):
     
     ndim = len(centroid)
     inside = False
     
-    Condition = [Conditioncheck(centroid, boxA, p, ndim) for p in range(0,ndim)]
+    Condition = [Conditioncheck(centroid, boxA, p, ndim, extent) for p in range(0,ndim)]
         
     inside = all(Condition)
     
     return inside
 
-def Conditioncheck(centroid, boxA, p, ndim):
+def Conditioncheck(centroid, boxA, p, ndim, extent):
     
       condition = False
-    
-      if centroid[p] >= boxA[p] and centroid[p] <= boxA[p + ndim]:
+     
+      vol = extent * ( boxA[p + ndim] - boxA[p] ) / 2
+      
+      if centroid[p] >=  boxA[p] - vol and centroid[p] <= boxA[p + ndim] + vol:
           
            condition = True
            
       return condition     
     
 
-def WatershedwithMask3D(Image, Label,mask, grid): 
+def WatershedwithMask3D(Image, Label,mask, grid, extent = 0): 
     properties = measure.regionprops(Label, Image) 
     binaryproperties = measure.regionprops(label(mask), Image) 
     
-    Image = np.add(Image, mask>0)
+    
     Coordinates = [prop.centroid for prop in properties] 
     BinaryCoordinates = [prop.centroid for prop in binaryproperties]
     
@@ -806,7 +808,7 @@ def WatershedwithMask3D(Image, Label,mask, grid):
             for i in range(0, len(Binarybbox)):
                 
                 box = Binarybbox[i]
-                inside = [iou3D(box, star) for star in Coordinates]
+                inside = [iou3D(box, star, extent) for star in Coordinates]
                 
                 if not any(inside) :
                          Coordinates.append(BinaryCoordinates[i])    
